
-- The MIT License (MIT)

-- Copyright (c) 2014 Dominoid

-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:

-- The above copyright notice notice shall be included in
-- all copies or substantial portions of the Software.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.
---------------------------------------------------------------------------------

--local WIZIT = ZO_Object:Subclass()
local ADDON_NAME = "DominoidsImmersiveThings"
local ADDON_VERSION = "v1.6.0"

local ns = 'DominoidsImmersiveThings'

-- Define some constants that will be used throughout the add-on.
local CONSTANT 	=
{
	AREA 		=	ZO_InteractWindowTargetArea,
	TITLE 		=	ZO_InteractWindowTargetAreaTitle,
	BODY 		=	ZO_InteractWindowTargetAreaBodyText,
	REWARD 		=	ZO_InteractWindowRewardAreaHeader,
	REWARDGOLD 	=	ZO_InteractWindowRewardAreaCurrency1,
	BG 			=	ZO_InteractWindowTopBG,
}

local wm = WINDOW_MANAGER
local em = EVENT_MANAGER

local strformat = string.format

local ZO_OptionsWindowSettingsScrollChild = ZO_OptionsWindowSettingsScrollChild
local ZO_OptionsWindow_InitializeControl = ZO_OptionsWindow_InitializeControl

local ZO_InteractWindowTargetAreaBodyText = ZO_InteractWindowTargetAreaBodyText
local ZO_InteractWindowRewardArea = ZO_InteractWindowRewardArea

local MOUSE_BUTTON_LEFT = 1
local MOUSE_BUTTON_RIGHT = 2
local MOUSE_BUTTON_MIDDLE = 3
local buttonCreated = 0 

local isThird = false
local intDelay = 0

local WIZIT = {}
WIZIT.name = "DominoidsImmersiveThings"

WIZIT.OnlyOnce = 0 

local CHATTER_ACCEPT_QUEST_BESTOWAL_ALT = 42
local CHATTER_COMPLETE_QUEST_CONFIRM_ALT = 43

local disabledColor = {
	ZO_DEFAULT_DISABLED_COLOR.r,
	ZO_DEFAULT_DISABLED_COLOR.g,
	ZO_DEFAULT_DISABLED_COLOR.b,
	ZO_DEFAULT_DISABLED_COLOR.a
}
local disabledIcon = {.75,.75,.75,1}

WIZIT.savedVarsVersion = 1

WIZIT.defaults = {
	showIcons = true,
	showNumbers = true,
	showAudioRepeat = true,
	highlightSpecial = true,
	hideDialog = true,
	hideStealthText = true,
	cameraSwitching = false,
	cameraDelay = 5000
}

local chatterIcons = {
	{
		CHATTER_START_COMPLETE_QUEST,
		CHATTER_ACCEPT_QUEST_BESTOWAL_ALT,
		texture = '/esoui/art/chatwindow/chat_notification_up.dds',
		secondary = {
			placeBehind = true,
			texture = '/esoui/art/chatwindow/chat_notification_over.dds',
			alpha = .9
		}
	},
	{
		CHATTER_COMPLETE_QUEST_CONFIRM_ALT,
		texture = '/esoui/art/buttons/accept_up.dds',
		color = {.5,1,0,1}, anchor = { RIGHT, LEFT, -4, 1 }
	},
	{
		CHATTER_START_NEW_QUEST_BESTOWAL,
		texture = '/esoui/art/inventory/inventory_tabicon_quest_up.dds',
		scale = 1.5, anchor = { RIGHT, LEFT, 2.5, 1 },
		secondary = {
			placeBehind = true,
			texture = '/esoui/art/inventory/inventory_tabicon_quest_over.dds',
			alpha = .7
		}
	},
	{
		CHATTER_START_TALK,
		texture = '/esoui/art/chatwindow/chat_notification_up.dds'
	},
	{
		CHATTER_TALK_CHOICE,
		CHATTER_TALK_CHOICE_PERSUADE_DISABLED,
		CHATTER_TALK_CHOICE_INTIMIDATE_DISABLED,
		texture = '/esoui/art/chatwindow/chat_notification_up.dds'
	},
	{
		CHATTER_GOODBYE,
		texture = '/esoui/art/buttons/decline_up.dds',
		scale = .7, anchor = { RIGHT, LEFT, -9, 1 }
	},
	{
		CHATTER_START_BUY_BAG_SPACE,
		CHATTER_PROMPT_BUY_BAG_SPACE,
		texture = '/esoui/art/mainmenu/menubar_inventory_up.dds',
		anchor = { RIGHT, LEFT, -4, 1 }
	},
	{
		CHATTER_TALK_CHOICE_MONEY,
		texture = '/esoui/art/bank/bank_tabicon_deposit_up.dds'
	},
	{
		CHATTER_START_STABLE,
		texture = '/esoui/art/mounts/tabicon_mounts_up.dds',
		scale = 1.2, anchor = { RIGHT, LEFT, -3, 1 }
	},
	{
		CHATTER_START_SHOP,
		CHATTER_START_TRADINGHOUSE,
		texture = '/esoui/art/guild/guildhistory_indexicon_guildstore_up.dds',
		anchor = { RIGHT, LEFT, -4, 1.5 }
	},
	{
		CHATTER_START_BANK,
		texture = '/esoui/art/guild/guildhistory_indexicon_guildbank_up.dds',
		anchor = { RIGHT, LEFT, -4, 0 }
	},
	{
		CHATTER_START_GUILDBANK,
		texture = '/esoui/art/guild/guildhistory_indexicon_guildbank_up.dds',
		anchor = { RIGHT, LEFT, -4, 0 }
	},
}

local memoizedChatterIcons = {}

function WIZIT.getChatterIcon(optionType)
	if not _G['USE_CHATTER_OPTION_ICON'] then return end
	
	local index = memoizedChatterIcons[optionType]
	if index ~= nil then return chatterIcons[index] end

	local l = #chatterIcons
	for i=1,l do
		local item = chatterIcons[i]

		for j,t in ipairs(item) do
			if t == optionType then
				memoizedChatterIcons[optionType] = i
				return item
			end
		end
	end
end

function WIZIT.chatterHandler()
	local numChildren = math.min(9, ZO_InteractWindowPlayerAreaOptions:GetNumChildren())

	for i=1,numChildren do
		local option = ZO_InteractWindowPlayerAreaOptions:GetChild(i)

		local text = option:GetText()

		if #text ~= 0 then
			local modifiedText = text

			-- highlight [Lie], [Persuade] etc.
			if option.enabled and WIZIT.savedVars.highlightSpecial then
				modifiedText = modifiedText:gsub('(%[.-%])', '|cffffff%1|r')
			end

			-- prepend numbers to chat options signifying related keyboard key (1-9)
			if WIZIT.savedVars.showNumbers then
				local prefix = strformat('%d. ', i)
				local from, to = text:find(prefix)

				if from ~= 1 then

					-- highlight shop type
					--[[if option.optionType == CHATTER_START_SHOP then
						modifiedText = modifiedText:gsub('(%(.-%))', '|cffffff%1|r')

					end]]--

					modifiedText = prefix..modifiedText
				end
			end

			option:SetText(modifiedText)
		end

		local icon = _G[option:GetName()..'Icon']
		local texture = _G[option:GetName()..'IconImage']

		-- make sure any previous changes to this icon do not stick
		if option.isTainted then
			icon:SetScale(1)
			
			icon:ClearAnchors()
			icon:SetAnchor(RIGHT, option, LEFT, -5, 0)

			texture:SetColor(1,1,1,1)
			texture:SetDesaturation(0)

			if option.secondary ~= nil then
				option.secondary:SetHidden(true)
				option.secondary:SetAlpha(1)
				option.secondary:SetColor(1,1,1,1)
				option.secondary:SetDesaturation(0)
			end

			option.isTainted = false
		end

		local custom = WIZIT.getChatterIcon(option.optionType)
		if custom ~= nil and WIZIT.savedVars.showIcons then

			icon:SetHidden(false)

			if custom.secondary ~= nil then
				if option.secondary == nil then
					option.secondary = WIZIT_DuplicateControl(WIZIT_createName(texture:GetName(), 'Secondary'), texture)
				end

				local secondary = option.secondary

				secondary:SetTexture(custom.secondary.texture)

				secondary:ClearAnchors()
				local point, relativeTo, relativePoint, offsetX, offsetY = select(2, texture:GetAnchor())
				if custom.secondary.offset ~= nil then
					local offsets = custom.secondary.offset
					offsetX = offsetX + offsets[1]
					offsetY = offsetY + offsets[2]
				end
				secondary:SetAnchor(point, relativeTo, relativePoint, offsetX, offsetY)

				secondary:SetScale(custom.secondary.scale or texture:GetScale())

				if custom.secondary.color ~= nil then
					secondary:SetColor(unpack(custom.secondary.color))
				end

				secondary:SetAlpha(custom.secondary.alpha or 1)

				if custom.secondary.placeBehind then
					secondary:SetDrawLevel(texture:GetDrawLevel() - 1)
				else
					secondary:SetDrawLevel(texture:GetDrawLevel() + 1)
				end

				secondary:SetHidden(false)
			end

			texture:SetTexture(custom.texture)

			if custom.scale ~= nil then
				icon:SetScale(custom.scale)
			end

			if custom.anchor ~= nil then
				local point, relativePoint, offsetX, offsetY = unpack(custom.anchor)
				icon:ClearAnchors()
				icon:SetAnchor(point, option, relativePoint, offsetX, offsetY)
			end

			if option.chosenBefore or not option.enabled then
				texture:SetDesaturation(1)
				texture:SetColor(unpack(disabledIcon))

				if option.secondary ~= nil then
					option.secondary:SetDesaturation(1)
					option.secondary:SetColor(unpack(disabledIcon))
				end
			elseif option.isImportant and not custom.colorOverride then
				texture:SetColor(1,0,0,1)

				if option.secondary ~= nil then
					option.secondary:SetColor(1,0,0,1)
				end
			elseif custom.color ~= nil then
				texture:SetColor(unpack(custom.color))
			end

			-- if guild bank or tradehouse and not part of any guild, dim label and icon
			if (option.optionType == CHATTER_START_GUILDBANK or option.optionType == CHATTER_START_TRADINGHOUSE) and GetNumGuilds() == 0 then
				option:SetColor(unpack(disabledColor))
				texture:SetDesaturation(1)
				texture:SetColor(unpack(disabledIcon))
			end

			option.isTainted = true
			texture:SetHidden(false)

		-- flag to avoid resetting unnecessarily
		else
			option.isTainted = false
			icon:SetHidden(true)
		end

	end

	if WIZIT.savedVars.showAudioRepeat and ZO_InteractWindowRewardArea:IsHidden() then
		local text = ZO_InteractWindowTargetAreaBodyText:GetText()

		-- assume non-NPC interaction thus no audio if the body text begins and ends with <> or quotes
		if (text:sub(1,1) == '<' and text:sub(-1) == '>') or (text:sub(1,1) == '"' and text:sub(-1) == '"') then
			WIZIT.audioRepeatControl:SetHidden(true)
		else
			WIZIT.audioRepeatControl:SetHidden(false)
		end
	else
		WIZIT.audioRepeatControl:SetHidden(true)
	end

end


-- Function that sets the initial styling (Fires when you speak to a NPC).
function WIZIT:Initialize()

	if WIZIT.savedVars.hideDialog then
		-- Hide the text
		CONSTANT.BODY:SetHidden( true )
		-- Clear any anchors already set on CONSTANT.TITLE
		CONSTANT.TITLE:ClearAnchors()
		-- Reposition the NPCs name to the bottom of CONSTANT.BODY and move it 1px up.
		CONSTANT.TITLE:SetAnchor( BOTTOMLEFT, CONSTANT.BODY, nil, 0, -1 )
		--WIZIT.initAudioRepeat()
		-- Grab all the other basic stylings.
		WIZIT:Style()
	else
		-- Show the body text.
		CONSTANT.BODY:SetHidden( false )
		-- Clear any anchors previously set on CONSTANT.TITLE
		CONSTANT.TITLE:ClearAnchors()
		-- Reposition the NPCs name to the top of CONSTANT.BODY and move it 45px up.
		CONSTANT.TITLE:SetAnchor( TOPLEFT, CONSTANT.BODY, nil, 0, -45 )
		WIZIT:Style()
	end

end

-- Function to toggle the body text.
function WIZND_Toggle()

	-- If the body text is hidden, show it and position the NPCs name accordingly.
	-- Else hide the body text and reposition the NPCs name.
	if 	CONSTANT.BODY:IsHidden() then
		-- Show the body text.
		CONSTANT.BODY:SetHidden( false )
		-- Clear any anchors previously set on CONSTANT.TITLE
		CONSTANT.TITLE:ClearAnchors()
		-- Reposition the NPCs name to the top of CONSTANT.BODY and move it 45px up.
		CONSTANT.TITLE:SetAnchor( TOPLEFT, CONSTANT.BODY, nil, 0, -45 )
		--Make toggle permanent
		WIZIT.savedVars.hideDialog = false
	else
		-- Hide the body text.
		CONSTANT.BODY:SetHidden( true )
		-- Clear any anchors previously set on CONSTANT.TITLE
		CONSTANT.TITLE:ClearAnchors()
		-- Reposition the NPCs name to the bottom of CONSTANT.BODY and move it 1px up.
		CONSTANT.TITLE:SetAnchor( BOTTOMLEFT, CONSTANT.BODY, nil, 0, -1 )
		--Make toggle permanent
		WIZIT.savedVars.hideDialog = true
	end

end

-- Function that sets the basic styling.
function WIZIT:Style( event )

	-- Remove the "-"s from the NPCs name.
	local NAME = ZO_InteractWindowTargetAreaTitle:GetText()
	CONSTANT.TITLE:SetText( string.sub(NAME, 2, -2) )
	-- Set the alignment of the NPCs name to the left.
	CONSTANT.TITLE:SetHorizontalAlignment( TEXT_ALIGN_LEFT )

	-- Push the top BG that is anchored to the title back, after moving the title.
	CONSTANT.BG:SetAnchor( TOPLEFT, CONSTANT.TITLE, TOPLEFT, -40, -120 )

	-- Clear any anchors already set on CONSTANT.BODY
	CONSTANT.BODY:ClearAnchors()
	-- Add 30px padding to the left of the body text.
	CONSTANT.BODY:SetAnchor( BOTTOMLEFT, CONSTANT.AREA, nil, 30, 0 )
	-- Push the body text 42px down.
	CONSTANT.BODY:SetAnchor( BOTTOMRIGHT, CONSTANT.AREA, nil, 0, 42 )

	-- Clear any anchors already set on CONSTANT.REWARD
	CONSTANT.REWARD:ClearAnchors()
	-- Add 40px padding to the left of the reward window.
	CONSTANT.REWARD:SetAnchor( BOTTOMLEFT, CONSTANT.BODY, BOTTOMLEFT, 10, 40 )

	-- Always show the body text if it ends with > or " (If we are examining a book, a piece of paper or a dead body etc)
	local TEXT = ZO_InteractWindowTargetAreaBodyText:GetText()
	if string.sub( TEXT, -1, -1 ) == ">" or string.sub( TEXT, -1, -1 ) == '"' then
		-- Show the body text.
		CONSTANT.BODY:SetHidden( false )
		-- Clear any anchors previously set on CONSTANT.TITLE
		CONSTANT.TITLE:ClearAnchors()
		-- Reposition the NPCs name to the top of CONSTANT.BODY and move it 45px up.
		CONSTANT.TITLE:SetAnchor( TOPLEFT, CONSTANT.BODY, nil, 0, -45 )
	end

	if event == EVENT_QUEST_COMPLETE_DIALOG then -- If we are completing a quest.
		-- Basic styling that covers most quest reward scenarios.
		ZO_InteractWindowRewardAreaCurrency1:ClearAnchors()
		ZO_InteractWindowRewardAreaCurrency1:SetAnchor( BOTTOMLEFT, ZO_InteractWindowRewardArea, nil, 40, 0 )
		if (ZO_InteractWindowRewardAreaCurrency2) then -- If the quest gives more than 1 currency.
			ZO_InteractWindowRewardAreaCurrency1:ClearAnchors()
			ZO_InteractWindowRewardAreaCurrency1:SetAnchor( BOTTOMLEFT, ZO_InteractWindowRewardArea, nil, 40, -25 )
		end
	end

end

function WIZIT.OnCombat() -- Player Combat State
	--d(IsUnitInCombat("player"))
	--d(isThird)
	if WIZIT.savedVars.cameraSwitching then 
		if IsUnitInCombat("player") then
			if isThird == false then
				ToggleGameCameraFirstPerson()
				isThird = true
			end
		else
			if isThird == true then
				intDelay = intDelay + 1
				zo_callLater(function()
					--This "intDelay" exists so that only the last call will fully execute the camera switch
					intDelay = intDelay - 1
					if IsUnitInCombat("player") then
					
					else
						if isThird == true and intDelay < 1 then
							ToggleGameCameraFirstPerson()
							isThird = false
						end
					end
				end, WIZIT.savedVars.cameraDelay)
			end
		end
	end
end

function WIZIT.removeStealthText(eventCode, unitTag, stealthState)
    if unitTag == "player" and WIZIT.savedVars.hideStealthText then 
        wm:GetControlByName("ZO_ReticleContainerStealthIconStealthText"):SetText("")
    end
end

function WIZIT.initStealthText(eventCode)
    WIZIT.removeStealthText(eventCode, "player", GetUnitStealthState("player"))
end

function WIZIT.initOptions()

--***************************************
	
	--// Settings panel LAM2
	local LAM2 = LibStub("LibAddonMenu-2.0")
	if ( not LAM2 ) then return end
	
	local panelData = {
			type = "panel",
			name = "Dominoid's Immersive Things",
			displayName = "Dominoid's Immersive Things",
			author = "Dominoid",
			version = ADDON_VERSION,
			slashCommand = "/jpgs",
			registerForRefresh = true,
			registerForDefaults = true,
	}
	LAM2:RegisterAddonPanel(ADDON_NAME, panelData)

	local optionsTable = {
		------------GENERAL--------------
		{
			type = "header",
			name = "Dialogue Settings",
			width = "full",
		},
		{
			type = "checkbox",
			name = "Show Repeat Button",
			tooltip = "Repeats or rotates voice over",
			getFunc = function() return WIZIT.savedVars.showAudioRepeat end,
			setFunc = function(var) WIZIT.savedVars.showAudioRepeat = var end,
			width = "full",
			default = true
		},
		{
			type = "checkbox",
			name = "Show Numbers",
			tooltip = "Show numbers toggle",
			getFunc = function() return WIZIT.savedVars.showNumbers end,
			setFunc = function(var) WIZIT.savedVars.showNumbers = var end,
			width = "full",
			default = true
		},
		{
			type = "checkbox",
			name = "Show Icons",
			tooltip = "Show icons toggle",
			getFunc = function() return WIZIT.savedVars.showIcons end,
			setFunc = function(var) WIZIT.savedVars.showIcons = var end,
			width = "full",
			default = true
		},
		{
			type = "checkbox",
			name = "Highlight Special",
			tooltip = "Makes all bracketed options ([Lie], [Persuade], etc.) stand out.",
			getFunc = function() return WIZIT.savedVars.highlightSpecial end,
			setFunc = function(var) WIZIT.savedVars.highlightSpecial = var end,
			width = "full",
			default = true
		},
		{
			type = "checkbox",
			name = "Hide Dialogue Text",
			tooltip = "Hide conversation dialogue",
			getFunc = function() return WIZIT.savedVars.hideDialog end,
			setFunc = function(var) WIZIT.savedVars.hideDialog = var end,
			width = "full",
			default = true
		},
		{
			type = "header",
			name = "Explore in First/Fight in Third",
			width = "full",
		},
		{
			type = "checkbox",
			name = "Enable Perspective Switching",
			tooltip = 'Enabling will switch camera view between first and third person view when going in and out of combat.',
			getFunc = function() return WIZIT.savedVars.cameraSwitching end,
			setFunc = function(var) WIZIT.savedVars.cameraSwitching = var end,
			width = "full",
			default = true
		},
		{
			type = "slider",
			name = "First Person Delay in Seconds",
			tooltip = "This is the delay when switching from third person back to first person once leaving combat.",
			min = 0, max = 10, step = 0.5,
			getFunc = function() return WIZIT.savedVars.cameraDelay / 1000 end,
			setFunc = function(newValue) WIZIT.savedVars.cameraDelay = newValue * 1000 end,
			default = 5000,
		},
		{
			type = "header",
			name = "Misc Immersion",
			width = "full",
		},
		{
			type = "checkbox",
			name = "Hide Stealth Text",
			tooltip = 'Removes the "HIDDEN" and "DETECTED" text when stealthing',
			getFunc = function() return WIZIT.savedVars.hideStealthText end,
			setFunc = function(var) WIZIT.savedVars.hideStealthText = var end,
			width = "full",
			default = true
		},

	}

	LAM2:RegisterOptionControls(ADDON_NAME, optionsTable)
	
	
--***************************************	
	
	
	
end

-- Function that registers all events and the keybinding.
function WIZIT:OnLoaded( eventCode, addOnName )
	--if addOnName ~= "DominoidsImmersiveThings" then return end
		if WIZIT.OnlyOnce == 1 then return end
	--if  addOnName = "DominoidsImmersiveThings"  then
		
		WIZIT.savedVars = ZO_SavedVars:NewAccountWide('DominoidsImmersiveThings_SavedVars', WIZIT.savedVarsVersion, 'Dialogue', WIZIT.defaults)

		_G['USE_CHATTER_OPTION_ICON'] = WIZIT.savedVars.showIcons

		WIZIT.initAudioRepeat()
		
		-- Register keybinding.
		ZO_CreateStringId( "SI_BINDING_NAME_TOGGLE_WIZND", "Toggle Dialogue Text" )
		em:RegisterForEvent( "WIZIT_Init", 		EVENT_CHATTER_BEGIN, 			function(event) WIZIT:Initialize() end )
		em:RegisterForEvent( "WIZIT_Update", 	EVENT_CONVERSATION_UPDATED,		function(event) WIZIT:Style(event) end )
		em:RegisterForEvent( "WIZIT_Complete", 	EVENT_QUEST_COMPLETE_DIALOG, 	function(event) WIZIT:Style(event) end )
		em:RegisterForEvent( "WIZIT_Offered", 	EVENT_QUEST_OFFERED, 			function(event) WIZIT:Style(event) end )
		
		em:RegisterForEvent(ns, EVENT_CHATTER_BEGIN, WIZIT.chatterHandler)
		em:RegisterForEvent(ns, EVENT_CONVERSATION_UPDATED, WIZIT.chatterHandler)
		em:RegisterForEvent(ns, EVENT_QUEST_OFFERED, WIZIT.chatterHandler)
		em:RegisterForEvent(ns, EVENT_QUEST_COMPLETE_DIALOG, WIZIT.chatterHandler)

		em:RegisterForEvent(ns, EVENT_STEALTH_STATE_CHANGED, WIZIT.removeStealthText)
		em:RegisterForEvent(ns, EVENT_RETICLE_HIDDEN_UPDATE, WIZIT.initStealthText)
		em:RegisterForEvent(ns, EVENT_RETICLE_TARGET_CHANGED, WIZIT.initStealthText)
		
		em:RegisterForEvent("ImmersionCamera", EVENT_PLAYER_COMBAT_STATE, WIZIT.OnCombat)
		
		EVENT_MANAGER:RegisterForEvent(ns, EVENT_PLAYER_ACTIVATED,
			function()
		
				WIZIT.initOptions()
				EVENT_MANAGER:UnregisterForEvent(ns, EVENT_PLAYER_ACTIVATED)
			end)

		--WIZIT = WIZIT.initOptions()
		
		WIZIT.OnlyOnce = 1
	--end
end

function WIZIT_createName(...)
	local str = table.concat({...}, '_')
	str = string.gsub(str, '[^%w_]', '_')

	local from, to = string.find(str, ns..'_')
	if from ~= 1 then
		str = ns..'_'..str
	end
	return str
end

function WIZIT.initAudioRepeat()
	if buttonCreated == 1 then return end
	buttonCreated = 1

	--WIZIT.audioRepeatControl = wm:CreateControl(WIZIT_createName('Dialogue', 'AudioRepeat'), ZO_InteractWindowTargetAreaTitle, CT_TEXTURE)
	WIZIT.audioRepeatControl = wm:CreateControl('Dialogue_Repeat', ZO_InteractWindowTargetAreaTitle, CT_TEXTURE)
	local audioIcon = WIZIT.audioRepeatControl

	--audioIcon:SetAnchor(LEFT, nil, LEFT, -.5, 0)
	audioIcon:SetAnchor(BOTTOMLEFT, CONSTANT.BODY, nil, -35, 0 )
	audioIcon:SetResizeToFitFile(true)
	audioIcon:SetAlpha(.5)
	audioIcon:SetMouseEnabled(true)

	local textures = {
		up = '/esoui/art/charactercreate/charactercreate_audio_up.dds',
		down = '/esoui/art/charactercreate/charactercreate_audio_down.dds'
	}

	audioIcon:SetTexture(textures.down) -- preload
	audioIcon:SetTexture(textures.up)

	audioIcon:SetHandler('OnMouseEnter', function(self)
		audioIcon:SetAlpha(1)

		if self.isMouseDown then
			self:SetTexture(textures.down)
		else
			self:SetTexture(textures.up)
		end
	end)
	audioIcon:SetHandler('OnMouseExit', function(self)
		self:SetTexture(textures.up)
		audioIcon:SetAlpha(.5)
	end)
	audioIcon:SetHandler('OnMouseDown', function(self, button)
		if wm:GetMouseOverControl() == self and button == MOUSE_BUTTON_LEFT then
			self.isMouseDown = true
			self:SetTexture(textures.down)
		end
	end)

	audioIcon:SetHandler('OnMouseUp', function(self, button)
		self.isMouseDown = false
		self:SetTexture(textures.up)

		-- clicked
		if wm:GetMouseOverControl() == self and button == MOUSE_BUTTON_LEFT then
			-- calling this function also triggers the playing of the dialogue audio
			-- does not work on reward screen
			-- rotates if NPC has multiple messages for the same instance (usually greetings)
			local text, numOptions, atGreeting = GetChatterData()

			-- if the text rotation winds up being different, replace existing text with new
			if ZO_InteractWindowTargetAreaBodyText:GetText() ~= text then
				ZO_InteractWindowTargetAreaBodyText:SetText(text)
			end
		end
	end)
end

-- Register the addon.
em:RegisterForEvent( "DominoidsImmersiveThings", EVENT_ADD_ON_LOADED, WIZIT.OnLoaded )